@@ -1,16 +1,7 @@
 from __future__ import annotations
 
 from functools import partial
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Container,
-    Iterable,
-    Mapping,
-    Protocol,
-    Sequence,
-    overload,
-)
+from typing import TYPE_CHECKING, Any, Protocol, overload
 
 from narwhals._compliant.typing import (
     CompliantExprT,
@@ -25,31 +16,42 @@ from narwhals._compliant.typing import (
     NativeFrameT_co,
     NativeSeriesT,
 )
-from narwhals.dependencies import is_numpy_array_2d
-from narwhals.utils import (
+from narwhals._utils import (
     exclude_column_names,
     get_column_names,
     passthrough_column_names,
 )
+from narwhals.dependencies import is_numpy_array_2d
 
 if TYPE_CHECKING:
+    from collections.abc import Container, Iterable, Sequence
+
     from typing_extensions import TypeAlias
 
     from narwhals._compliant.selectors import CompliantSelectorNamespace
     from narwhals._compliant.when_then import CompliantWhen, EagerWhen
-    from narwhals.dtypes import DType
-    from narwhals.schema import Schema
-    from narwhals.typing import ConcatMethod, Into1DArray, NonNestedLiteral, _2DArray
-    from narwhals.utils import Implementation, Version
+    from narwhals._utils import Implementation, Version
+    from narwhals.typing import (
+        ConcatMethod,
+        Into1DArray,
+        IntoDType,
+        IntoSchema,
+        NonNestedLiteral,
+        _2DArray,
+    )
 
     Incomplete: TypeAlias = Any
 
-__all__ = ["CompliantNamespace", "EagerNamespace"]
+__all__ = [
+    "CompliantNamespace",
+    "DepthTrackingNamespace",
+    "EagerNamespace",
+    "LazyNamespace",
+]
 
 
 class CompliantNamespace(Protocol[CompliantFrameT, CompliantExprT]):
     _implementation: Implementation
-    _backend_version: tuple[int, ...]
     _version: Version
 
     def all(self) -> CompliantExprT:
@@ -69,11 +71,13 @@ class CompliantNamespace(Protocol[CompliantFrameT, CompliantExprT]):
         return self._expr.from_column_indices(*column_indices, context=self)
 
     def len(self) -> CompliantExprT: ...
-    def lit(
-        self, value: NonNestedLiteral, dtype: DType | type[DType] | None
+    def lit(self, value: NonNestedLiteral, dtype: IntoDType | None) -> CompliantExprT: ...
+    def all_horizontal(
+        self, *exprs: CompliantExprT, ignore_nulls: bool
+    ) -> CompliantExprT: ...
+    def any_horizontal(
+        self, *exprs: CompliantExprT, ignore_nulls: bool
     ) -> CompliantExprT: ...
-    def all_horizontal(self, *exprs: CompliantExprT) -> CompliantExprT: ...
-    def any_horizontal(self, *exprs: CompliantExprT) -> CompliantExprT: ...
     def sum_horizontal(self, *exprs: CompliantExprT) -> CompliantExprT: ...
     def mean_horizontal(self, *exprs: CompliantExprT) -> CompliantExprT: ...
     def min_horizontal(self, *exprs: CompliantExprT) -> CompliantExprT: ...
@@ -91,6 +95,7 @@ class CompliantNamespace(Protocol[CompliantFrameT, CompliantExprT]):
     def selectors(self) -> CompliantSelectorNamespace[Any, Any]: ...
     @property
     def _expr(self) -> type[CompliantExprT]: ...
+    def coalesce(self, *exprs: CompliantExprT) -> CompliantExprT: ...
 
 
 class DepthTrackingNamespace(
@@ -119,21 +124,28 @@ class LazyNamespace(
     CompliantNamespace[CompliantLazyFrameT, LazyExprT],
     Protocol[CompliantLazyFrameT, LazyExprT, NativeFrameT_co],
 ):
+    @property
+    def _backend_version(self) -> tuple[int, ...]:
+        return self._implementation._backend_version()
+
     @property
     def _lazyframe(self) -> type[CompliantLazyFrameT]: ...
 
     def from_native(self, data: NativeFrameT_co | Any, /) -> CompliantLazyFrameT:
         if self._lazyframe._is_native(data):
             return self._lazyframe.from_native(data, context=self)
-        else:  # pragma: no cover
-            msg = f"Unsupported type: {type(data).__name__!r}"
-            raise TypeError(msg)
+        msg = f"Unsupported type: {type(data).__name__!r}"  # pragma: no cover
+        raise TypeError(msg)
 
 
 class EagerNamespace(
     DepthTrackingNamespace[EagerDataFrameT, EagerExprT],
     Protocol[EagerDataFrameT, EagerSeriesT, EagerExprT, NativeFrameT, NativeSeriesT],
 ):
+    @property
+    def _backend_version(self) -> tuple[int, ...]:
+        return self._implementation._backend_version()
+
     @property
     def _dataframe(self) -> type[EagerDataFrameT]: ...
     @property
@@ -146,20 +158,12 @@ class EagerNamespace(
     def from_native(self, data: NativeFrameT, /) -> EagerDataFrameT: ...
     @overload
     def from_native(self, data: NativeSeriesT, /) -> EagerSeriesT: ...
-    # TODO @dangotbanned: Align `PandasLike` typing with `_namespace`, then drop this `@overload`
-    # - Using the guards there introduces `_NativeModin`, `_NativeCuDF`
-    # - These types haven't been integrated into the backend
-    # - Most of the `pandas` stuff is still untyped
-    @overload
-    def from_native(
-        self, data: NativeFrameT | NativeSeriesT | Any, /
-    ) -> EagerDataFrameT | EagerSeriesT: ...
     def from_native(
         self, data: NativeFrameT | NativeSeriesT | Any, /
     ) -> EagerDataFrameT | EagerSeriesT:
         if self._dataframe._is_native(data):
             return self._dataframe.from_native(data, context=self)
-        elif self._series._is_native(data):
+        if self._series._is_native(data):
             return self._series.from_native(data, context=self)
         msg = f"Unsupported type: {type(data).__name__!r}"
         raise TypeError(msg)
@@ -169,17 +173,14 @@ class EagerNamespace(
 
     @overload
     def from_numpy(
-        self,
-        data: _2DArray,
-        /,
-        schema: Mapping[str, DType] | Schema | Sequence[str] | None,
+        self, data: _2DArray, /, schema: IntoSchema | Sequence[str] | None
     ) -> EagerDataFrameT: ...
 
     def from_numpy(
         self,
         data: Into1DArray | _2DArray,
         /,
-        schema: Mapping[str, DType] | Schema | Sequence[str] | None = None,
+        schema: IntoSchema | Sequence[str] | None = None,
     ) -> EagerDataFrameT | EagerSeriesT:
         if is_numpy_array_2d(data):
             return self._dataframe.from_numpy(data, schema=schema, context=self)
