@@ -1,8 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-from typing import Generic
-from typing import TypeVar
+from typing import TYPE_CHECKING, Generic, TypeVar
 
 if TYPE_CHECKING:
     from narwhals.expr import Expr
@@ -17,9 +15,6 @@ class ExprStringNamespace(Generic[ExprT]):
     def len_chars(self) -> ExprT:
         r"""Return the length of each string as the number of characters.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -41,7 +36,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |└───────┴───────────┘|
             └─────────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.len_chars()
         )
 
@@ -56,9 +51,6 @@ class ExprStringNamespace(Generic[ExprT]):
             literal: Treat `pattern` as a literal string.
             n: Number of matches to replace.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -73,7 +65,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |1  abc abc123   abc123|
             └──────────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.replace(
                 pattern, value, literal=literal, n=n
             )
@@ -87,9 +79,6 @@ class ExprStringNamespace(Generic[ExprT]):
             value: String that will replace the matched substring.
             literal: Treat `pattern` as a literal string.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -104,7 +93,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |1  abc abc123      123|
             └──────────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.replace_all(
                 pattern, value, literal=literal
             )
@@ -119,9 +108,6 @@ class ExprStringNamespace(Generic[ExprT]):
                 If set to None (default), all leading and trailing whitespace is removed
                 instead.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -132,7 +118,7 @@ class ExprStringNamespace(Generic[ExprT]):
             ... )
             {'fruits': ['apple', '\nmango'], 'stripped': ['apple', 'mango']}
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.strip_chars(characters)
         )
 
@@ -142,9 +128,6 @@ class ExprStringNamespace(Generic[ExprT]):
         Arguments:
             prefix: prefix substring
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -160,7 +143,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |2   None       None|
             └───────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.starts_with(prefix)
         )
 
@@ -170,9 +153,6 @@ class ExprStringNamespace(Generic[ExprT]):
         Arguments:
             suffix: suffix substring
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -188,7 +168,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |2   None       None|
             └───────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.ends_with(suffix)
         )
 
@@ -200,9 +180,6 @@ class ExprStringNamespace(Generic[ExprT]):
             literal: If True, treats the pattern as a literal string.
                      If False, assumes the pattern is a regular expression.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -221,7 +198,7 @@ class ExprStringNamespace(Generic[ExprT]):
             default_match: [[true,false,true]]
             case_insensitive_match: [[true,false,true]]
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.contains(
                 pattern, literal=literal
             )
@@ -235,9 +212,6 @@ class ExprStringNamespace(Generic[ExprT]):
             length: Length of the slice. If set to `None` (default), the slice is taken to the
                 end of the string.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -253,7 +227,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |2  papaya       ya|
             └──────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.slice(
                 offset=offset, length=length
             )
@@ -265,9 +239,6 @@ class ExprStringNamespace(Generic[ExprT]):
         Arguments:
             by: Substring to split by.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -288,7 +259,7 @@ class ExprStringNamespace(Generic[ExprT]):
             |└─────────┴────────────────┘|
             └────────────────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.split(by=by)
         )
 
@@ -298,9 +269,6 @@ class ExprStringNamespace(Generic[ExprT]):
         Arguments:
             n: Number of elements to take. Negative indexing is **not** supported.
 
-        Returns:
-            A new expression.
-
         Notes:
             If the length of the string has fewer than `n` characters, the full string is returned.
 
@@ -317,7 +285,7 @@ class ExprStringNamespace(Generic[ExprT]):
             lyrics: [["taata","taatatata","zukkyun"]]
             lyrics_head: [["taata","taata","zukky"]]
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.slice(0, n)
         )
 
@@ -327,9 +295,6 @@ class ExprStringNamespace(Generic[ExprT]):
         Arguments:
             n: Number of elements to take. Negative indexing is **not** supported.
 
-        Returns:
-            A new expression.
-
         Notes:
             If the length of the string has fewer than `n` characters, the full string is returned.
 
@@ -346,7 +311,7 @@ class ExprStringNamespace(Generic[ExprT]):
             lyrics: [["taata","taatatata","zukkyun"]]
             lyrics_tail: [["taata","atata","kkyun"]]
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.slice(
                 offset=-n, length=None
             )
@@ -370,9 +335,6 @@ class ExprStringNamespace(Generic[ExprT]):
             format: Format to use for conversion. If set to None (default), the format is
                 inferred from the data.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -393,16 +355,42 @@ class ExprStringNamespace(Generic[ExprT]):
             |└─────────────────────┘|
             └───────────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.to_datetime(format=format)
         )
 
+    def to_date(self, format: str | None = None) -> ExprT:
+        """Convert to date dtype.
+
+        Warning:
+            As different backends auto-infer format in different ways, if `format=None`
+            there is no guarantee that the result will be equal.
+
+        Arguments:
+            format: Format to use for conversion. If set to None (default), the format is inferred from the data.
+
+        Examples:
+            >>> import pyarrow as pa
+            >>> import narwhals as nw
+            >>> df_native = pa.table({"a": ["2020-01-01", "2020-01-02"]})
+            >>> df = nw.from_native(df_native)
+            >>> df.select(nw.col("a").str.to_date(format="%Y-%m-%d"))
+            ┌────────────────────────────┐
+            |     Narwhals DataFrame     |
+            |----------------------------|
+            |pyarrow.Table               |
+            |a: date32[day]              |
+            |----                        |
+            |a: [[2020-01-01,2020-01-02]]|
+            └────────────────────────────┘
+        """
+        return self._expr._with_elementwise(
+            lambda plx: self._expr._to_compliant_expr(plx).str.to_date(format=format)
+        )
+
     def to_uppercase(self) -> ExprT:
         r"""Transform string to uppercase variant.
 
-        Returns:
-            A new expression.
-
         Notes:
             The PyArrow backend will convert 'ß' to 'ẞ' instead of 'SS'.
             For more info see [the related issue](https://github.com/apache/arrow/issues/34599).
@@ -422,16 +410,13 @@ class ExprStringNamespace(Generic[ExprT]):
             |1   None      None|
             └──────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.to_uppercase()
         )
 
     def to_lowercase(self) -> ExprT:
         r"""Transform string to lowercase variant.
 
-        Returns:
-            A new expression.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -446,6 +431,34 @@ class ExprStringNamespace(Generic[ExprT]):
             |1   None      None|
             └──────────────────┘
         """
-        return self._expr._with_callable(
+        return self._expr._with_elementwise(
             lambda plx: self._expr._to_compliant_expr(plx).str.to_lowercase()
         )
+
+    def zfill(self, width: int) -> ExprT:
+        """Transform string to zero-padded variant.
+
+        Arguments:
+            width: The desired length of the string after padding. If the length of the
+                string is greater than `width`, no padding is applied.
+                If `width` is less than 0, no padding is applied.
+
+        Examples:
+            >>> import pandas as pd
+            >>> import narwhals as nw
+            >>> df_native = pd.DataFrame({"digits": ["+1", "-1", "1", None]})
+            >>> df = nw.from_native(df_native)
+            >>> df.with_columns(zfill_col=nw.col("digits").str.zfill(3))
+            ┌──────────────────┐
+            |Narwhals DataFrame|
+            |------------------|
+            |  digits zfill_col|
+            |0     +1       +01|
+            |1     -1       -01|
+            |2      1       001|
+            |3   None      None|
+            └──────────────────┘
+        """
+        return self._expr._with_elementwise(
+            lambda plx: self._expr._to_compliant_expr(plx).str.zfill(width)
+        )
