@@ -8,11 +8,12 @@ from __future__ import annotations
 
 from collections import OrderedDict
 from functools import partial
-from typing import TYPE_CHECKING, Iterable, Mapping, cast
+from typing import TYPE_CHECKING, cast
 
-from narwhals.utils import Implementation, Version, parse_version
+from narwhals._utils import Implementation, Version
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable, Mapping
     from typing import Any, ClassVar
 
     import polars as pl
@@ -21,15 +22,11 @@ if TYPE_CHECKING:
     from narwhals.dtypes import DType
     from narwhals.typing import DTypeBackend
 
-    BaseSchema = OrderedDict[str, DType]
-else:
-    # Python 3.8 does not support generic OrderedDict at runtime
-    BaseSchema = OrderedDict
 
 __all__ = ["Schema"]
 
 
-class Schema(BaseSchema):
+class Schema(OrderedDict[str, "DType"]):
     """Ordered mapping of column names to their data type.
 
     Arguments:
@@ -68,35 +65,20 @@ class Schema(BaseSchema):
         super().__init__(schema)
 
     def names(self) -> list[str]:
-        """Get the column names of the schema.
-
-        Returns:
-            Column names.
-        """
+        """Get the column names of the schema."""
         return list(self.keys())
 
     def dtypes(self) -> list[DType]:
-        """Get the data types of the schema.
-
-        Returns:
-            Data types of schema.
-        """
+        """Get the data types of the schema."""
         return list(self.values())
 
     def len(self) -> int:
-        """Get the number of columns in the schema.
-
-        Returns:
-            Number of columns.
-        """
+        """Get the number of columns in the schema."""
         return len(self)
 
     def to_arrow(self) -> pa.Schema:
         """Convert Schema to a pyarrow Schema.
 
-        Returns:
-            A pyarrow Schema.
-
         Examples:
             >>> import narwhals as nw
             >>> schema = nw.Schema({"a": nw.Int64(), "b": nw.Datetime("ns")})
@@ -122,9 +104,6 @@ class Schema(BaseSchema):
             dtype_backend: Backend(s) used for the native types. When providing more than
                 one, the length of the iterable must be equal to the length of the schema.
 
-        Returns:
-            An ordered mapping of column names to their pandas data type.
-
         Examples:
             >>> import narwhals as nw
             >>> schema = nw.Schema({"a": nw.Int64(), "b": nw.Datetime("ns")})
@@ -134,14 +113,11 @@ class Schema(BaseSchema):
             >>> schema.to_pandas("pyarrow")
             {'a': 'Int64[pyarrow]', 'b': 'timestamp[ns][pyarrow]'}
         """
-        import pandas as pd  # ignore-banned-import
-
         from narwhals._pandas_like.utils import narwhals_to_native_dtype
 
         to_native_dtype = partial(
             narwhals_to_native_dtype,
             implementation=Implementation.PANDAS,
-            backend_version=parse_version(pd),
             version=self._version,
         )
         if dtype_backend is None or isinstance(dtype_backend, str):
@@ -149,38 +125,32 @@ class Schema(BaseSchema):
                 name: to_native_dtype(dtype=dtype, dtype_backend=dtype_backend)
                 for name, dtype in self.items()
             }
-        else:
-            backends = tuple(dtype_backend)
-            if len(backends) != len(self):
-                from itertools import chain, islice, repeat
+        backends = tuple(dtype_backend)
+        if len(backends) != len(self):
+            from itertools import chain, islice, repeat
 
-                n_user, n_actual = len(backends), len(self)
-                suggestion = tuple(
-                    islice(
-                        chain.from_iterable(islice(repeat(backends), n_actual)), n_actual
-                    )
-                )
-                msg = (
-                    f"Provided {n_user!r} `dtype_backend`(s), but schema contains {n_actual!r} field(s).\n"
-                    "Hint: instead of\n"
-                    f"    schema.to_pandas({backends})\n"
-                    "you may want to use\n"
-                    f"    schema.to_pandas({backends[0]})\n"
-                    f"or\n"
-                    f"    schema.to_pandas({suggestion})"
-                )
-                raise ValueError(msg)
-            return {
-                name: to_native_dtype(dtype=dtype, dtype_backend=backend)
-                for name, dtype, backend in zip(self.keys(), self.values(), backends)
-            }
+            n_user, n_actual = len(backends), len(self)
+            suggestion = tuple(
+                islice(chain.from_iterable(islice(repeat(backends), n_actual)), n_actual)
+            )
+            msg = (
+                f"Provided {n_user!r} `dtype_backend`(s), but schema contains {n_actual!r} field(s).\n"
+                "Hint: instead of\n"
+                f"    schema.to_pandas({backends})\n"
+                "you may want to use\n"
+                f"    schema.to_pandas({backends[0]})\n"
+                f"or\n"
+                f"    schema.to_pandas({suggestion})"
+            )
+            raise ValueError(msg)
+        return {
+            name: to_native_dtype(dtype=dtype, dtype_backend=backend)
+            for name, dtype, backend in zip(self.keys(), self.values(), backends)
+        }
 
     def to_polars(self) -> pl.Schema:
         """Convert Schema to a polars Schema.
 
-        Returns:
-            A polars Schema or plain dict (prior to polars 1.0).
-
         Examples:
             >>> import narwhals as nw
             >>> schema = nw.Schema({"a": nw.Int64(), "b": nw.Datetime("ns")})
@@ -191,14 +161,9 @@ class Schema(BaseSchema):
 
         from narwhals._polars.utils import narwhals_to_native_dtype
 
-        pl_version = parse_version(pl)
+        pl_version = Implementation.POLARS._backend_version()
         schema = (
-            (
-                name,
-                narwhals_to_native_dtype(
-                    dtype, self._version, backend_version=pl_version
-                ),
-            )
+            (name, narwhals_to_native_dtype(dtype, self._version))
             for name, dtype in self.items()
         )
         return (
