@@ -2,20 +2,22 @@ from __future__ import annotations
 
 import math
 from collections.abc import Iterator, Mapping, Sequence
-from typing import TYPE_CHECKING, Any, Callable, Generic, Literal, overload
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Generic, Literal, overload
 
 from narwhals._utils import (
+    Implementation,
+    Version,
     _validate_rolling_arguments,
     ensure_type,
     generate_repr,
     is_compliant_series,
+    is_eager_allowed,
     is_index_selector,
-    parse_version,
     supports_arrow_c_stream,
 )
-from narwhals.dependencies import is_numpy_scalar
-from narwhals.dtypes import _validate_dtype
-from narwhals.exceptions import ComputeError
+from narwhals.dependencies import is_numpy_array_1d, is_numpy_scalar
+from narwhals.dtypes import _validate_dtype, _validate_into_dtype
+from narwhals.exceptions import ComputeError, InvalidOperationError
 from narwhals.series_cat import SeriesCatNamespace
 from narwhals.series_dt import SeriesDateTimeNamespace
 from narwhals.series_list import SeriesListNamespace
@@ -33,7 +35,6 @@ if TYPE_CHECKING:
     from typing_extensions import Self
 
     from narwhals._compliant import CompliantSeries
-    from narwhals._utils import Implementation
     from narwhals.dataframe import DataFrame, MultiIndexSelector
     from narwhals.dtypes import DType
     from narwhals.typing import (
@@ -71,6 +72,8 @@ class Series(Generic[IntoSeriesT]):
             ```
     """
 
+    _version: ClassVar[Version] = Version.MAIN
+
     @property
     def _dataframe(self) -> type[DataFrame[Any]]:
         from narwhals.dataframe import DataFrame
@@ -89,6 +92,71 @@ class Series(Generic[IntoSeriesT]):
             msg = f"Expected Polars Series or an object which implements `__narwhals_series__`, got: {type(series)}."
             raise AssertionError(msg)
 
+    @classmethod
+    def from_numpy(
+        cls,
+        name: str,
+        values: _1DArray,
+        dtype: IntoDType | None = None,
+        *,
+        backend: ModuleType | Implementation | str,
+    ) -> Series[Any]:
+        """Construct a Series from a NumPy ndarray.
+
+        Arguments:
+            name: Name of resulting Series.
+            values: One-dimensional data represented as a NumPy ndarray.
+            dtype: (Narwhals) dtype. If not provided, the native library
+                may auto-infer it from `values`.
+            backend: specifies which eager backend instantiate to.
+
+                `backend` can be specified in various ways
+
+                - As `Implementation.<BACKEND>` with `BACKEND` being `PANDAS`, `PYARROW`,
+                    `POLARS`, `MODIN` or `CUDF`.
+                - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
+                - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
+
+        Examples:
+            >>> import numpy as np
+            >>> import polars as pl
+            >>> import narwhals as nw
+            >>>
+            >>> arr = np.arange(5, 10)
+            >>> nw.Series.from_numpy("arr", arr, dtype=nw.Int8, backend="polars")
+            ┌──────────────────┐
+            | Narwhals Series  |
+            |------------------|
+            |shape: (5,)       |
+            |Series: 'arr' [i8]|
+            |[                 |
+            |        5         |
+            |        6         |
+            |        7         |
+            |        8         |
+            |        9         |
+            |]                 |
+            └──────────────────┘
+        """
+        if not is_numpy_array_1d(values):
+            msg = "`from_numpy` only accepts 1D numpy arrays"
+            raise ValueError(msg)
+        if dtype:
+            _validate_into_dtype(dtype)
+        implementation = Implementation.from_backend(backend)
+        if is_eager_allowed(implementation):
+            ns = cls._version.namespace.from_backend(implementation).compliant
+            compliant = ns.from_numpy(values).alias(name)
+            if dtype:
+                return cls(compliant.cast(dtype), level="full")
+            return cls(compliant, level="full")
+        msg = (
+            f"{implementation} support in Narwhals is lazy-only, but `Series.from_numpy` is an eager-only function.\n\n"
+            "Hint: you may want to use an eager backend and then call `.lazy`, e.g.:\n\n"
+            f"    nw.Series.from_numpy(arr, backend='pyarrow').to_frame().lazy('{implementation}')"
+        )
+        raise ValueError(msg)
+
     @property
     def implementation(self) -> Implementation:
         """Return implementation of native Series.
@@ -96,9 +164,6 @@ class Series(Generic[IntoSeriesT]):
         This can be useful when you need to use special-casing for features outside of
         Narwhals' scope - for example, when dealing with pandas' Period Dtype.
 
-        Returns:
-            Implementation.
-
         Examples:
             >>> import narwhals as nw
             >>> import pandas as pd
@@ -202,11 +267,11 @@ class Series(Generic[IntoSeriesT]):
         if supports_arrow_c_stream(native_series):
             return native_series.__arrow_c_stream__(requested_schema=requested_schema)
         try:
-            import pyarrow as pa  # ignore-banned-import
+            pa_version = Implementation.PYARROW._backend_version()
         except ModuleNotFoundError as exc:  # pragma: no cover
             msg = f"'pyarrow>=16.0.0' is required for `Series.__arrow_c_stream__` for object of type {type(native_series)}"
             raise ModuleNotFoundError(msg) from exc
-        if parse_version(pa) < (16, 0):  # pragma: no cover
+        if pa_version < (16, 0):  # pragma: no cover
             msg = f"'pyarrow>=16.0.0' is required for `Series.__arrow_c_stream__` for object of type {type(native_series)}"
             raise ModuleNotFoundError(msg)
         from narwhals._arrow.utils import chunked_array
@@ -217,9 +282,6 @@ class Series(Generic[IntoSeriesT]):
     def to_native(self) -> IntoSeriesT:
         """Convert Narwhals series to native series.
 
-        Returns:
-            Series of class that user started with.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -243,9 +305,6 @@ class Series(Generic[IntoSeriesT]):
             indices: Position(s) to set items at.
             values: Values to set.
 
-        Returns:
-            A new Series with values set at given positions.
-
         Note:
             This method always returns a new Series, without modifying the original one.
             Using this function in a for-loop is an anti-pattern, we recommend building
@@ -289,9 +348,6 @@ class Series(Generic[IntoSeriesT]):
     def shape(self) -> tuple[int]:
         """Get the shape of the Series.
 
-        Returns:
-            A tuple containing the length of the Series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -315,9 +371,6 @@ class Series(Generic[IntoSeriesT]):
     def pipe(self, function: Callable[[Any], Self], *args: Any, **kwargs: Any) -> Self:
         """Pipe function call.
 
-        Returns:
-            A new Series with the results of the piped function applied.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -345,9 +398,6 @@ class Series(Generic[IntoSeriesT]):
 
         Null values count towards the total.
 
-        Returns:
-            The number of elements in the Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -362,9 +412,6 @@ class Series(Generic[IntoSeriesT]):
     def dtype(self) -> DType:
         """Get the data type of the Series.
 
-        Returns:
-            The data type of the Series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -379,9 +426,6 @@ class Series(Generic[IntoSeriesT]):
     def name(self) -> str:
         """Get the name of the Series.
 
-        Returns:
-            The name of the Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -437,9 +481,6 @@ class Series(Generic[IntoSeriesT]):
                   $1-\alpha$ and $1$ if `adjust=True`,
                   and $1-\alpha$ and $\alpha$ if `adjust=False`.
 
-        Returns:
-            Series
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -471,9 +512,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             dtype: Data type that the object will be cast into.
 
-        Returns:
-            A new Series with the specified data type.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -495,9 +533,6 @@ class Series(Generic[IntoSeriesT]):
     def to_frame(self) -> DataFrame[Any]:
         """Convert to dataframe.
 
-        Returns:
-            A DataFrame containing this Series as a single column.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -525,9 +560,6 @@ class Series(Generic[IntoSeriesT]):
             your original dataframe, so we recommend only calling this
             when you absolutely need to.
 
-        Returns:
-            A list of Python objects.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -541,9 +573,6 @@ class Series(Generic[IntoSeriesT]):
     def mean(self) -> float:
         """Reduce this Series to the mean value.
 
-        Returns:
-            The average of all elements in the Series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -560,9 +589,6 @@ class Series(Generic[IntoSeriesT]):
         Notes:
             Results might slightly differ across backends due to differences in the underlying algorithms used to compute the median.
 
-        Returns:
-            The median value of all elements in the Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -576,9 +602,6 @@ class Series(Generic[IntoSeriesT]):
     def skew(self) -> float | None:
         """Calculate the sample skewness of the Series.
 
-        Returns:
-            The sample skewness of the Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -599,9 +622,6 @@ class Series(Generic[IntoSeriesT]):
         Kurtosis is the fourth central moment divided by the square of the variance.
         The Fisher's definition is used where 3.0 is subtracted from the result to give 0.0 for a normal distribution.
 
-        Returns:
-            The kurtosis (Fisher's definition) without bias correction of the column.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -615,9 +635,6 @@ class Series(Generic[IntoSeriesT]):
     def count(self) -> int:
         """Returns the number of non-null elements in the Series.
 
-        Returns:
-            The number of non-null elements in the Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -636,9 +653,6 @@ class Series(Generic[IntoSeriesT]):
         Notes:
             Only works on Series of data type Boolean.
 
-        Returns:
-            A boolean indicating if any values in the Series are True.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -654,9 +668,6 @@ class Series(Generic[IntoSeriesT]):
 
         If there are no non-null elements, the result is `True`.
 
-        Returns:
-            A boolean indicating if all values in the Series are True.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -670,9 +681,6 @@ class Series(Generic[IntoSeriesT]):
     def min(self) -> Any:
         """Get the minimal value in this Series.
 
-        Returns:
-            The minimum value in the Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -686,9 +694,6 @@ class Series(Generic[IntoSeriesT]):
     def max(self) -> Any:
         """Get the maximum value in this Series.
 
-        Returns:
-            The maximum value in the Series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -730,9 +735,6 @@ class Series(Generic[IntoSeriesT]):
 
         If there are no non-null elements, the result is zero.
 
-        Returns:
-            The sum of all elements in the Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -748,10 +750,7 @@ class Series(Generic[IntoSeriesT]):
 
         Arguments:
             ddof: "Delta Degrees of Freedom": the divisor used in the calculation is N - ddof,
-                     where N represents the number of elements.
-
-        Returns:
-            The standard deviation of all elements in the Series.
+                    where N represents the number of elements.
 
         Examples:
             >>> import polars as pl
@@ -768,7 +767,7 @@ class Series(Generic[IntoSeriesT]):
 
         Arguments:
             ddof: "Delta Degrees of Freedom": the divisor used in the calculation is N - ddof,
-                     where N represents the number of elements.
+                    where N represents the number of elements.
 
         Examples:
             >>> import pyarrow as pa
@@ -791,9 +790,6 @@ class Series(Generic[IntoSeriesT]):
             lower_bound: Lower bound value.
             upper_bound: Upper bound value.
 
-        Returns:
-            A new Series with values clipped to the specified bounds.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -821,9 +817,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             other: Sequence of primitive type.
 
-        Returns:
-            A new Series with boolean values indicating if the elements are in the other sequence.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -847,9 +840,6 @@ class Series(Generic[IntoSeriesT]):
     def arg_true(self) -> Self:
         """Find elements where boolean Series is True.
 
-        Returns:
-            A new Series with the indices of elements that are True.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -875,9 +865,6 @@ class Series(Generic[IntoSeriesT]):
             See [null_handling](../concepts/null_handling.md/)
             for reference.
 
-        Returns:
-            A new Series with null values removed.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -895,9 +882,6 @@ class Series(Generic[IntoSeriesT]):
     def abs(self) -> Self:
         """Calculate the absolute value of each element.
 
-        Returns:
-            A new Series with the absolute values of the original elements.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -923,9 +907,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             reverse: reverse the operation
 
-        Returns:
-            A new Series with the cumulative sum of non-null values.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -947,9 +928,6 @@ class Series(Generic[IntoSeriesT]):
                 expensive to compute. Settings this to `True` blocks the possibility
                 to run on the streaming engine for Polars.
 
-        Returns:
-            A new Series with duplicate values removed.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -983,9 +961,6 @@ class Series(Generic[IntoSeriesT]):
 
                 s.diff().fill_null(0).cast(nw.Int64)
 
-        Returns:
-            A new Series with the difference between each element and its predecessor.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1012,9 +987,6 @@ class Series(Generic[IntoSeriesT]):
             n: Number of indices to shift forward. If a negative value is passed,
                 values are shifted in the opposite direction instead.
 
-        Returns:
-            A new Series with values shifted by n positions.
-
         Notes:
             pandas may change the dtype here, for example when introducing missing
             values in an integer column. To ensure, that the dtype doesn't change,
@@ -1056,9 +1028,6 @@ class Series(Generic[IntoSeriesT]):
             seed: Seed for the random number generator. If set to None (default), a random
                 seed is generated for each sample operation.
 
-        Returns:
-            A new Series containing randomly sampled values from the original Series.
-
         Notes:
             The `sample` method returns a Series with a specified number of
             randomly selected items chosen from this Series.
@@ -1113,9 +1082,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             name: The new name.
 
-        Returns:
-            A new Series with the updated name.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1156,9 +1122,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             name: The new name.
 
-        Returns:
-            A new Series with the updated name.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1196,9 +1159,6 @@ class Series(Generic[IntoSeriesT]):
                 (default), the data type is determined automatically based on the other
                 inputs.
 
-        Returns:
-            A new Series with values replaced according to the mapping.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1232,9 +1192,6 @@ class Series(Generic[IntoSeriesT]):
             descending: Sort in descending order.
             nulls_last: Place null values last instead of first.
 
-        Returns:
-            A new sorted Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1263,9 +1220,6 @@ class Series(Generic[IntoSeriesT]):
             See [null_handling](../concepts/null_handling.md/)
             for reference.
 
-        Returns:
-            A boolean Series indicating which values are null.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1288,9 +1242,6 @@ class Series(Generic[IntoSeriesT]):
     def is_nan(self) -> Self:
         """Returns a boolean Series indicating which values are NaN.
 
-        Returns:
-            A boolean Series indicating which values are NaN.
-
         Notes:
             pandas handles null values differently from Polars and PyArrow.
             See [null_handling](../concepts/null_handling.md/)
@@ -1329,9 +1280,6 @@ class Series(Generic[IntoSeriesT]):
             - For pandas Series of `object` dtype, `fill_null` will not automatically change the
               Series' dtype as pandas used to do. Explicitly call `cast` if you want the dtype to change.
 
-        Returns:
-            A new Series with null values filled according to the specified value or strategy.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1386,9 +1334,6 @@ class Series(Generic[IntoSeriesT]):
             If the value of the `lower_bound` is greater than that of the `upper_bound`,
             then the values will be False, as no value can satisfy the condition.
 
-        Returns:
-            A boolean Series indicating which values are between the given bounds.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1418,9 +1363,6 @@ class Series(Generic[IntoSeriesT]):
     def n_unique(self) -> int:
         """Count the number of unique values.
 
-        Returns:
-            Number of unique values in the Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1434,9 +1376,6 @@ class Series(Generic[IntoSeriesT]):
     def to_numpy(self) -> _1DArray:
         """Convert to numpy.
 
-        Returns:
-            NumPy ndarray representation of the Series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1450,9 +1389,6 @@ class Series(Generic[IntoSeriesT]):
     def to_pandas(self) -> pd.Series[Any]:
         """Convert to pandas Series.
 
-        Returns:
-            A pandas Series containing the data from this Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1469,9 +1405,6 @@ class Series(Generic[IntoSeriesT]):
     def to_polars(self) -> pl.Series:
         """Convert to polars Series.
 
-        Returns:
-            A polars Series containing the data from this Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1617,9 +1550,6 @@ class Series(Generic[IntoSeriesT]):
     def filter(self, predicate: Any) -> Self:
         """Filter elements in the Series based on a condition.
 
-        Returns:
-            A new Series with elements that satisfy the condition.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1640,9 +1570,6 @@ class Series(Generic[IntoSeriesT]):
     def is_duplicated(self) -> Self:
         r"""Get a mask of all duplicated rows in the Series.
 
-        Returns:
-            A new Series with boolean values indicating duplicated rows.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1661,14 +1588,11 @@ class Series(Generic[IntoSeriesT]):
               ]
             ]
         """
-        return ~self.is_unique()
+        return self._with_compliant(self._compliant_series.is_duplicated())
 
     def is_empty(self) -> bool:
         r"""Check if the series is empty.
 
-        Returns:
-            A boolean indicating if the series is empty.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1681,14 +1605,11 @@ class Series(Generic[IntoSeriesT]):
             >>> s_nw.filter(s_nw > 10).is_empty()
             True
         """
-        return self._compliant_series.len() == 0
+        return self._compliant_series.is_empty()
 
     def is_unique(self) -> Self:
         r"""Get a mask of all unique rows in the Series.
 
-        Returns:
-            A new Series with boolean values indicating unique rows.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1711,9 +1632,6 @@ class Series(Generic[IntoSeriesT]):
             See [null_handling](../concepts/null_handling.md/)
             for reference.
 
-        Returns:
-            The number of null values in the Series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1727,9 +1645,6 @@ class Series(Generic[IntoSeriesT]):
     def is_first_distinct(self) -> Self:
         r"""Return a boolean mask indicating the first occurrence of each distinct value.
 
-        Returns:
-            A new Series with boolean values indicating the first occurrence of each distinct value.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1753,9 +1668,6 @@ class Series(Generic[IntoSeriesT]):
     def is_last_distinct(self) -> Self:
         r"""Return a boolean mask indicating the last occurrence of each distinct value.
 
-        Returns:
-            A new Series with boolean values indicating the last occurrence of each distinct value.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1777,9 +1689,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             descending: Check if the Series is sorted in descending order.
 
-        Returns:
-            A boolean indicating if the Series is sorted.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1813,9 +1722,6 @@ class Series(Generic[IntoSeriesT]):
                 defaults to "proportion", otherwise defaults to "count".
             normalize: If true gives relative frequencies of the unique values
 
-        Returns:
-            A DataFrame with two columns
-
                 - The original values as first column
                 - Either count or proportion as second column, depending on normalize parameter.
 
@@ -1851,9 +1757,6 @@ class Series(Generic[IntoSeriesT]):
             quantile: Quantile between 0.0 and 1.0.
             interpolation: Interpolation method.
 
-        Returns:
-            The quantile value.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1880,9 +1783,6 @@ class Series(Generic[IntoSeriesT]):
             mask: Boolean Series
             other: Series of same type.
 
-        Returns:
-            A new Series with values selected from self or other based on the mask.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1918,9 +1818,6 @@ class Series(Generic[IntoSeriesT]):
         If no index is provided, this is equivalent to `s[0]`, with a check
         that the shape is (1,). With an index, this is equivalent to `s[index]`.
 
-        Returns:
-            The scalar value of the Series or the element at the given index.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -1939,9 +1836,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             n: Number of rows to return.
 
-        Returns:
-            A new Series containing the first n rows.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -1961,9 +1855,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             n: Number of rows to return.
 
-        Returns:
-            A new Series with the last n rows.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -1988,9 +1879,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             decimals: Number of decimals to round by.
 
-        Returns:
-            A new Series with rounded values.
-
         Notes:
             For values exactly halfway between rounded decimal values pandas behaves differently than Polars and Arrow.
 
@@ -2025,9 +1913,6 @@ class Series(Generic[IntoSeriesT]):
             separator: Separator/delimiter used when generating column names.
             drop_first: Remove the first category from the variable being encoded.
 
-        Returns:
-            A new DataFrame containing the dummy/indicator variables.
-
         Notes:
             pandas and Polars handle null values differently. Polars distinguishes
             between NaN and Null, whereas pandas doesn't.
@@ -2063,9 +1948,6 @@ class Series(Generic[IntoSeriesT]):
             n: Gather every *n*-th row.
             offset: Starting index.
 
-        Returns:
-            A new Series with every nth value starting from the offset.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -2089,9 +1971,6 @@ class Series(Generic[IntoSeriesT]):
     def to_arrow(self) -> pa.Array[Any]:
         r"""Convert to arrow.
 
-        Returns:
-            A PyArrow Array containing the data from the Series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -2115,9 +1994,6 @@ class Series(Generic[IntoSeriesT]):
 
         Can return multiple values.
 
-        Returns:
-            A new Series containing the mode(s) (values that appear most frequently).
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2137,9 +2013,6 @@ class Series(Generic[IntoSeriesT]):
             False for NaN and Null's in the Dask and pandas non-nullable backend, while
             for Polars, PyArrow and pandas nullable backends null values are kept as such.
 
-        Returns:
-            Expression of `Boolean` data type.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -2166,9 +2039,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             reverse: reverse the operation
 
-        Returns:
-            A new Series with the cumulative count of non-null values.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -2194,9 +2064,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             reverse: reverse the operation
 
-        Returns:
-            A new Series with the cumulative min of non-null values.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2217,9 +2084,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             reverse: reverse the operation
 
-        Returns:
-            A new Series with the cumulative max of non-null values.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -2237,7 +2101,6 @@ class Series(Generic[IntoSeriesT]):
                 3
               ]
             ]
-
         """
         return self._with_compliant(self._compliant_series.cum_max(reverse=reverse))
 
@@ -2247,9 +2110,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             reverse: reverse the operation
 
-        Returns:
-            A new Series with the cumulative product of non-null values.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -2289,9 +2149,6 @@ class Series(Generic[IntoSeriesT]):
                 less than or equal to `window_size`
             center: Set the labels at the center of the window.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2339,9 +2196,6 @@ class Series(Generic[IntoSeriesT]):
                 less than or equal to `window_size`
             center: Set the labels at the center of the window.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -2399,9 +2253,6 @@ class Series(Generic[IntoSeriesT]):
             center: Set the labels at the center of the window.
             ddof: Delta Degrees of Freedom; the divisor for a length N window is N - ddof.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import polars as pl
             >>> import narwhals as nw
@@ -2458,9 +2309,6 @@ class Series(Generic[IntoSeriesT]):
             center: Set the labels at the center of the window.
             ddof: Delta Degrees of Freedom; the divisor for a length N window is N - ddof.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2518,9 +2366,6 @@ class Series(Generic[IntoSeriesT]):
 
             descending: Rank in descending order.
 
-        Returns:
-            A new series with rank data as values.
-
         Examples:
             >>> import pyarrow as pa
             >>> import narwhals as nw
@@ -2554,7 +2399,7 @@ class Series(Generic[IntoSeriesT]):
 
     def hist(
         self,
-        bins: list[float | int] | None = None,
+        bins: list[float] | None = None,
         *,
         bin_count: int | None = None,
         include_breakpoint: bool = True,
@@ -2570,9 +2415,6 @@ class Series(Generic[IntoSeriesT]):
             bin_count: If no bins provided, this will be used to determine the distance of the bins.
             include_breakpoint: Include a column that shows the intervals as categories.
 
-        Returns:
-            A new DataFrame containing the counts of values that occur within each passed bin.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2588,24 +2430,25 @@ class Series(Generic[IntoSeriesT]):
             |3        8.00      2|
             └────────────────────┘
         """
-        if bins is not None and bin_count is not None:
-            msg = "can only provide one of `bin_count` or `bins`"
-            raise ComputeError(msg)
-        if bins is None and bin_count is None:
-            bin_count = 10  # polars (v1.20) sets bin=10 if neither are provided.
-
         if bins is not None:
-            for i in range(1, len(bins)):
-                if bins[i - 1] >= bins[i]:
-                    msg = "bins must increase monotonically"
-                    raise ComputeError(msg)
+            if any(bins[i - 1] >= bins[i] for i in range(1, len(bins))):
+                msg = "bins must increase monotonically"
+                raise ComputeError(msg)
+            if bin_count is not None:
+                msg = f"can only provide one of `bin_count` or `bins`, got: {bin_count=}, {bins=}"
+                raise ComputeError(msg)
+            result = self._compliant_series.hist_from_bins(
+                bins=bins, include_breakpoint=include_breakpoint
+            )
+        else:
+            # polars (v1.20) sets bin=10 if neither are provided.
+            default = 10
+            bin_count = default if bin_count is None else bin_count
+            result = self._compliant_series.hist_from_bin_count(
+                bin_count=bin_count, include_breakpoint=include_breakpoint
+            )
 
-        return self._dataframe(
-            self._compliant_series.hist(
-                bins=bins, bin_count=bin_count, include_breakpoint=include_breakpoint
-            ),
-            level=self._level,
-        )
+        return self._dataframe(result, level=self._level)
 
     def log(self, base: float = math.e) -> Self:
         r"""Compute the logarithm to a given base.
@@ -2613,9 +2456,6 @@ class Series(Generic[IntoSeriesT]):
         Arguments:
             base: Given base, defaults to `e`
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2636,9 +2476,6 @@ class Series(Generic[IntoSeriesT]):
     def exp(self) -> Self:
         r"""Compute the exponent.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2659,9 +2496,6 @@ class Series(Generic[IntoSeriesT]):
     def sqrt(self) -> Self:
         r"""Compute the square root.
 
-        Returns:
-            A new series.
-
         Examples:
             >>> import pandas as pd
             >>> import narwhals as nw
@@ -2679,6 +2513,79 @@ class Series(Generic[IntoSeriesT]):
         """
         return self._with_compliant(self._compliant_series.sqrt())
 
+    def is_close(
+        self,
+        other: Self | NumericLiteral,
+        *,
+        abs_tol: float = 0.0,
+        rel_tol: float = 1e-09,
+        nans_equal: bool = False,
+    ) -> Self:
+        r"""Get a boolean mask of the values being close to the other values.
+
+        Two values `a` and `b` are considered close if the following condition holds:
+
+        $$
+        |a-b| \le max \{ \text{rel\_tol} \cdot max \{ |a|, |b| \}, \text{abs\_tol} \}
+        $$
+
+        Arguments:
+            other: Values to compare with.
+            abs_tol: Absolute tolerance. This is the maximum allowed absolute difference
+                between two values. Must be non-negative.
+            rel_tol: Relative tolerance. This is the maximum allowed difference between
+                two values, relative to the larger absolute value. Must be in the range
+                [0, 1).
+            nans_equal: Whether NaN values should be considered equal.
+
+        Notes:
+            The implementation of this method is symmetric and mirrors the behavior of
+            `math.isclose`. Specifically note that this behavior is different to
+            `numpy.isclose`.
+
+        Examples:
+            >>> import pyarrow as pa
+            >>> import narwhals as nw
+            >>>
+            >>> data = [1.0, float("inf"), 1.41, None, float("nan")]
+            >>> s_native = pa.chunked_array([data])
+            >>> s = nw.from_native(s_native, series_only=True)
+            >>> s.is_close(1.4, abs_tol=0.1).to_native()  # doctest:+ELLIPSIS
+            <pyarrow.lib.ChunkedArray object at ...>
+            [
+              [
+                false,
+                false,
+                true,
+                null,
+                false
+              ]
+            ]
+        """
+        if not self.dtype.is_numeric():
+            msg = (
+                f"is_close operation not supported for dtype `{self.dtype}`\n\n"
+                "Hint: `is_close` is only supported for numeric types"
+            )
+            raise InvalidOperationError(msg)
+
+        if abs_tol < 0:
+            msg = f"`abs_tol` must be non-negative but got {abs_tol}"
+            raise ComputeError(msg)
+
+        if not (0 <= rel_tol < 1):
+            msg = f"`rel_tol` must be in the range [0, 1) but got {rel_tol}"
+            raise ComputeError(msg)
+
+        return self._with_compliant(
+            self._compliant_series.is_close(
+                self._extract_native(other),
+                abs_tol=abs_tol,
+                rel_tol=rel_tol,
+                nans_equal=nans_equal,
+            )
+        )
+
     @property
     def str(self) -> SeriesStringNamespace[Self]:
         return SeriesStringNamespace(self)
