@@ -1,7 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-from typing import Generic
+from typing import TYPE_CHECKING, Generic
 
 from narwhals.typing import SeriesT
 
@@ -16,9 +15,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def date(self) -> SeriesT:
         """Get the date in a datetime series.
 
-        Returns:
-            A new Series with the date portion of the datetime values.
-
         Raises:
             NotImplementedError: If pandas default backend is being used.
 
@@ -42,9 +38,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def year(self) -> SeriesT:
         """Get the year in a datetime series.
 
-        Returns:
-            A new Series containing the year component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import polars as pl
@@ -66,9 +59,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def month(self) -> SeriesT:
         """Gets the month in a datetime series.
 
-        Returns:
-            A new Series containing the month component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import polars as pl
@@ -90,9 +80,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def day(self) -> SeriesT:
         """Extracts the day in a datetime series.
 
-        Returns:
-            A new Series containing the day component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pyarrow as pa
@@ -117,9 +104,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def hour(self) -> SeriesT:
         """Extracts the hour in a datetime series.
 
-        Returns:
-            A new Series containing the hour component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pyarrow as pa
@@ -144,9 +128,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def minute(self) -> SeriesT:
         """Extracts the minute in a datetime series.
 
-        Returns:
-            A new Series containing the minute component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -167,9 +148,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def second(self) -> SeriesT:
         """Extracts the seconds in a datetime series.
 
-        Returns:
-            A new Series containing the second component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -190,9 +168,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def millisecond(self) -> SeriesT:
         """Extracts the milliseconds in a datetime series.
 
-        Returns:
-            A new Series containing the millisecond component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -216,9 +191,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def microsecond(self) -> SeriesT:
         """Extracts the microseconds in a datetime series.
 
-        Returns:
-            A new Series containing the microsecond component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -234,7 +206,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             0    400000
             1         0
             Name: datetime, dtype: int32
-
         """
         return self._narwhals_series._with_compliant(
             self._narwhals_series._compliant_series.dt.microsecond()
@@ -243,9 +214,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def nanosecond(self) -> SeriesT:
         """Extract the nanoseconds in a date series.
 
-        Returns:
-            A new Series containing the nanosecond component of each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -269,9 +237,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def ordinal_day(self) -> SeriesT:
         """Get ordinal day.
 
-        Returns:
-            A new Series containing the ordinal day (day of year) for each datetime value.
-
         Examples:
             >>> from datetime import datetime
             >>> import pyarrow as pa
@@ -296,9 +261,7 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def weekday(self) -> SeriesT:
         """Extract the week day in a datetime series.
 
-        Returns:
-            A new Series containing the week day for each datetime value.
-            Returns the ISO weekday number where monday = 1 and sunday = 7
+        Note that Monday = 1 and Sunday = 7.
 
         Examples:
             >>> from datetime import datetime
@@ -329,9 +292,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             however, pandas may change the dtype to float when there are missing values,
             consider using `fill_null()` in this case.
 
-        Returns:
-            A new Series containing the total number of minutes for each timedelta value.
-
         Examples:
             >>> from datetime import timedelta
             >>> import polars as pl
@@ -360,9 +320,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             however, pandas may change the dtype to float when there are missing values,
             consider using `fill_null()` in this case.
 
-        Returns:
-            A new Series containing the total number of seconds for each timedelta value.
-
         Examples:
             >>> from datetime import timedelta
             >>> import polars as pl
@@ -391,9 +348,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             however, pandas may change the dtype to float when there are missing values,
             consider using `fill_null()` in this case.
 
-        Returns:
-            A new Series containing the total number of milliseconds for each timedelta value.
-
         Examples:
             >>> from datetime import timedelta
             >>> import polars as pl
@@ -420,9 +374,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
     def total_microseconds(self) -> SeriesT:
         """Get total microseconds.
 
-        Returns:
-            A new Series containing the total number of microseconds for each timedelta value.
-
         Notes:
             The function outputs the total microseconds in the int dtype by default,
             however, pandas may change the dtype to float when there are missing values,
@@ -459,9 +410,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             however, pandas may change the dtype to float when there are missing values,
             consider using `fill_null()` in this case.
 
-        Returns:
-            A new Series containing the total number of nanoseconds for each timedelta value.
-
         Examples:
             >>> from datetime import datetime
             >>> import polars as pl
@@ -488,9 +436,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
         Arguments:
             format: Format string for converting the datetime to string.
 
-        Returns:
-            A new Series with the datetime values formatted as strings according to the specified format.
-
         Notes:
             Unfortunately, different libraries interpret format directives a bit
             differently.
@@ -527,12 +472,7 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
             >>> import pyarrow as pa
             >>> import narwhals as nw
             >>> s_native = pa.chunked_array(
-            ...     [
-            ...         [
-            ...             datetime(2020, 3, 1),
-            ...             datetime(2020, 4, 1),
-            ...         ]
-            ...     ]
+            ...     [[datetime(2020, 3, 1), datetime(2020, 4, 1)]]
             ... )
             >>> s = nw.from_native(s_native, series_only=True)
             >>> s.dt.to_string("%Y/%m/%d").to_native()  # doctest: +ELLIPSIS
@@ -554,9 +494,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
         Arguments:
             time_zone: Target time zone.
 
-        Returns:
-            A new Series with the specified time zone.
-
         Examples:
             >>> from datetime import datetime, timezone
             >>> import polars as pl
@@ -591,9 +528,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
         Arguments:
             time_zone: Target time zone.
 
-        Returns:
-            A new Series with the specified time zone.
-
         Examples:
             >>> from datetime import datetime, timezone
             >>> import pandas as pd
@@ -626,9 +560,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
                 - 'us': microsecond.
                 - 'ms': millisecond.
 
-        Returns:
-            A new Series with timestamps in the specified time unit.
-
         Examples:
             >>> from datetime import date
             >>> import pandas as pd
@@ -671,9 +602,6 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
                 - 'q': quarter.
                 - 'y': year.
 
-        Returns:
-            Series of data type `Date` or `Datetime`.
-
         Examples:
             >>> from datetime import datetime
             >>> import pandas as pd
@@ -687,3 +615,35 @@ class SeriesDateTimeNamespace(Generic[SeriesT]):
         return self._narwhals_series._with_compliant(
             self._narwhals_series._compliant_series.dt.truncate(every)
         )
+
+    def offset_by(self, by: str) -> SeriesT:
+        """Offset this date by a relative time offset.
+
+        Arguments:
+            by: The offset. Must be of form `<multiple><unit>`,
+                where `multiple` is a positive integer and `unit` is one of
+
+                - 'ns': nanosecond.
+                - 'us': microsecond.
+                - 'ms': millisecond.
+                - 's': second.
+                - 'm': minute.
+                - 'h': hour.
+                - 'd': day.
+                - 'mo': month.
+                - 'q': quarter.
+                - 'y': year.
+
+        Examples:
+            >>> from datetime import datetime
+            >>> import pandas as pd
+            >>> import narwhals as nw
+            >>> s_native = pd.Series([datetime(2021, 3, 1, 12, 34)])
+            >>> s = nw.from_native(s_native, series_only=True)
+            >>> s.dt.offset_by("1h").to_native()
+            0   2021-03-01 13:34:00
+            dtype: datetime64[ns]
+        """
+        return self._narwhals_series._with_compliant(
+            self._narwhals_series._compliant_series.dt.offset_by(by)
+        )
