@@ -2,16 +2,15 @@ from __future__ import annotations
 
 from functools import wraps
 from typing import TYPE_CHECKING, Any, Callable, Literal, cast, overload
-from warnings import warn
 
 import narwhals as nw
 from narwhals import exceptions, functions as nw_f
+from narwhals._exceptions import issue_warning
 from narwhals._typing_compat import TypeVar, assert_never
 from narwhals._utils import (
     Implementation,
     Version,
     deprecate_native_namespace,
-    find_stacklevel,
     generate_temporary_column_name,
     inherit_doc,
     is_ordered_categorical,
@@ -288,6 +287,8 @@ class LazyFrame(NwLazyFrame[IntoFrameT]):
 
 
 class Series(NwSeries[IntoSeriesT]):
+    _version = Version.V1
+
     @inherit_doc(NwSeries)
     def __init__(
         self, series: Any, *, level: Literal["full", "lazy", "interchange"]
@@ -298,6 +299,18 @@ class Series(NwSeries[IntoSeriesT]):
     # We need to override any method which don't return Self so that type
     # annotations are correct.
 
+    @classmethod
+    def from_numpy(
+        cls,
+        name: str,
+        values: _1DArray,
+        dtype: IntoDType | None = None,
+        *,
+        backend: ModuleType | Implementation | str,
+    ) -> Series[Any]:
+        result = super().from_numpy(name, values, dtype, backend=backend)
+        return cast("Series[Any]", result)
+
     @property
     def _dataframe(self) -> type[DataFrame[Any]]:
         return DataFrame
@@ -326,14 +339,13 @@ class Series(NwSeries[IntoSeriesT]):
         bin_count: int | None = None,
         include_breakpoint: bool = True,
     ) -> DataFrame[Any]:
-        from narwhals._utils import find_stacklevel
         from narwhals.exceptions import NarwhalsUnstableWarning
 
         msg = (
             "`Series.hist` is being called from the stable API although considered "
             "an unstable feature."
         )
-        warn(message=msg, category=NarwhalsUnstableWarning, stacklevel=find_stacklevel())
+        issue_warning(msg, NarwhalsUnstableWarning)
         return _stableify(
             super().hist(
                 bins=bins, bin_count=bin_count, include_breakpoint=include_breakpoint
@@ -375,7 +387,7 @@ class Expr(NwExpr):
                 "`maintain_order` has no effect and is only kept around for backwards-compatibility. "
                 "You can safely remove this argument."
             )
-            warn(message=msg, category=UserWarning, stacklevel=find_stacklevel())
+            issue_warning(msg, UserWarning)
         return self._with_filtration(lambda plx: self._to_compliant_expr(plx).unique())
 
     def sort(self, *, descending: bool = False, nulls_last: bool = False) -> Self:
@@ -420,9 +432,6 @@ class Expr(NwExpr):
             with_replacement: Allow values to be sampled more than once.
             seed: Seed for the random number generator. If set to None (default), a random
                 seed is generated for each sample operation.
-
-        Returns:
-            A new expression.
         """
         return self._with_filtration(
             lambda plx: self._to_compliant_expr(plx).sample(  # type: ignore[attr-defined]
@@ -1030,9 +1039,8 @@ def narwhalify(
 
     if func is None:
         return decorator
-    else:
-        # If func is not None, it means the decorator is used without arguments
-        return decorator(func)
+    # If func is not None, it means the decorator is used without arguments
+    return decorator(func)
 
 
 def all() -> Expr:
